# ---------------------------------------------------------------------------
# BadLinkFinder - A Plugin for Movable Type
#
# Release 0.50
# August 8, 2008
#
# Author: nlog(n)
# Description: http://nlogn.ath.cx/plugins/badlinkfinder/
# Email: n [at] nlogn [dot] ath [dot] cx
# ---------------------------------------------------------------------------

package MT::Plugin::BadLinkFinder;

use strict;
use LWP::UserAgent;
use HTML::TreeBuilder;
use HTML::Entities qw(encode_entities decode_entities);
use Encode qw(encode decode);
use MT::ConfigMgr;
use MT::Template::Context;
use base qw(MT::Plugin);

our $Name = 'BadLinkFinder';
our $VERSION = '0.50';

my $plugin = new MT::Plugin::BadLinkFinder({
    name => $Name,
    version => $VERSION,
    author_name => 'nlog(n)',
    author_link => 'http://nlogn.ath.cx/',
    plugin_link => 'http://nlogn.ath.cx/plugins/badlinkfinder/',
    description => '<MT_TRANS phrase="Adds a class named &quot;badlink&quot; to the anchor tag when its reference URL returns error status code.">',
    settings => new MT::PluginSettings([
        ['exclude', { Default => '' }],
    ]),
    blog_config_template => \&blog_config_template,
    l10n_class => 'BadLinkFinder::L10N',
});
MT->add_plugin($plugin);

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        tags => {
            help_url => 'http://nlogn.ath.cx/plugins/%t/',
            block => {
                'BadLinkFinder' => \&BadLinkFinder,
            },
        }
    });
}

sub blog_config_template {
  my ($plugin, $param, undef) = @_;
  return <<'TMPL';
<mt:App:Setting
    id="exclude"
    label="<__trans phrase="Exclude">">
    <div class="textarea-wrapper">
        <input name="exclude" id="exclude" class="full-width" size="30" value="<TMPL_VAR NAME=EXCLUDE ESCAPE=HTML>" />
    </div>
</mt:App:Setting>
TMPL
}

sub BadLinkFinder {
  my ($ctx, $args) = @_;

  my $tokens = $ctx->stash('tokens');
  my $builder = $ctx->stash('builder');

  defined(my $out = $builder->build($ctx, $tokens))
    or return $ctx->error($builder->errstr);

  return $out if (defined($ctx->stash('entry')) && $ctx->stash('entry')->to_ping_urls);

  my $charset = {
       'UTF-8' => 'utf8',
       'EUC-JP' => 'euc-jp',
       'Shift_JIS' => 'shiftjis',
       'ISO-2022-JP' => 'iso-2022-jp'
     }->{MT::ConfigMgr->instance->PublishCharset} || 'utf8';
  my $entry_url = defined $ctx->stash('entry') ? $ctx->stash('entry')->permalink : undef;
  my $exclude = $args->{exclude} || $ctx->stash('blog')->site_url;

  $out = encode_entities($out, '&');
  $out = decode($charset, $out);
  my $tree = HTML::TreeBuilder->new();
  $tree->handler(process => sub {$_[0]->process(substr($_[1], 2, -1));}, 'self, text');
  $tree->attr_encoded(1);
  $tree->store_comments(1);
  $tree->store_declarations(1);
  $tree->store_pis(1);
  $tree->xml_pic(1);
  $tree->parse($out);
  $tree->eof;
  $tree = linkcheck($tree, $entry_url, $exclude);
  $out = '';
  foreach my $c ($tree->guts()) {
    $out .= ref $c ? $c->as_HTML('<>&"', '  ', {}) : encode_entities($c, '&');
  }
  $tree->delete;
  $out = encode($charset, decode_entities($out));
  $out;
}

sub linkcheck {
  my ($tree, $entry_url, $exclude) = @_;
  my ($a_clone, $url, %urls, $res);
  my @excludes = split /\|/, $exclude;

  foreach my $a ($tree->find_by_tag_name('a')) {
    $a_clone = $a->clone();
    $url = defragment($a_clone->attr('href'));
    if ($url =~ m|^http://|
     && $url ne $entry_url
     && not grep {$url =~ m|$_|} @excludes) {
      if (! found_word_in_keys($url, \%urls)) {
        my $ua = LWP::UserAgent->new;
        $ua->agent(__PACKAGE__ . '/' . $VERSION);
        $ua->parse_head(0);
        $ua->timeout(5);
        my $req = HTTP::Request->new(HEAD => $url);
        $res = $ua->request($req);
        $urls{$url} = $res;
      } else {
        $res = $urls{$url};
      }
      if ($res->is_error) {
        my $attr = $a_clone->attr('class');
        $attr = add_class('badlink', $attr);
        $a_clone->attr('class' => $attr) if $attr;
      }
    }
    $a->replace_with($a_clone)->delete;
  }
  return $tree;
}

sub defragment {
  my $url = shift;
  my @a = split('#', $url);
  return $a[0];
}

sub found_word_in_keys {
  my ($word, $hash) = @_;
  return $hash->{$word};
}

sub add_class {
  my ($class, $attr) = @_;
  if ($attr) {
    my @attrs = split(/\s+/, $attr);
    $attr .= in_array($class, \@attrs) ? '' : ' ' . $class;
  } else {
    $attr = $class;
  }
  return $attr;
}

sub in_array {
  my ($search_for, $array) = @_;
  return grep {$search_for eq $_} @$array;
}

1;
